@@ -77,7 +77,8 @@ def arrayuniqify(X, retainorder=False):
 
 def recarrayuniqify(X, retainorder=False):
     """
-    Very fast uniqify routine for numpy record arrays.
+    Very fast uniqify routine for numpy record arrays (or ndarrays with 
+    structured dtype).
 
     Record array version of func:`tabular.fast.arrayuniqify`.
 
@@ -192,7 +193,8 @@ def equalspairs(X, Y):
 
 def recarrayequalspairs(X,Y,weak=True):
     """
-    Indices of elements in a sorted numpy recarray equal to those in another.
+    Indices of elements in a sorted numpy recarray (or ndarray with 
+    structured dtype) equal to those in another.
 
     Record array version of func:`tabular.fast.equalspairs`, but slightly 
     different because the concept of being sorted is less well-defined for a 
@@ -305,7 +307,8 @@ def isin(X,Y):
 
 def recarrayisin(X,Y,weak=True):
     """
-    Indices of elements in a numpy array that appear in another.
+    Indices of elements in a numpy record array (or ndarray with structured 
+    dtype) that appear in another.
 
     Fast routine for determining indices of elements in numpy record array `X` 
     that appear in numpy record array `Y`, returning a boolean array `Z` such 
@@ -390,7 +393,8 @@ def arraydifference(X,Y):
 
 def recarraydifference(X,Y):
     """
-    Records of a numpy recarray that do not appear in another.
+    Records of a numpy recarray (or ndarray with structured dtype)
+    that do not appear in another.
 
     Fast routine for determining which records in numpy array `X`
     do not appear in numpy recarray `Y`.
