@@ -28,8 +28,8 @@ __all__ = ['loadSV', 'loadSVcols','loadSVrecs', 'saveSV', 'loadbinary',
 
 DEFAULT_VERBOSITY=5
 
-def loadSV(fname, names=None, dtype=None, shape=None, formats=None, titles=None, 
-                  aligned=False, byteorder=None, **kwargs):
+def loadSV(fname, names=None, dtype=None, shape=None, formats=None,    
+           titles=None, aligned=False, byteorder=None,  **kwargs):
 
     """
     Load a delimited text file to a numpy record array.
@@ -103,10 +103,9 @@ def loadSV(fname, names=None, dtype=None, shape=None, formats=None, titles=None,
     if names is None and 'names' in metadata.keys():
 	      names = metadata['names']
 
-    return [np.rec.fromarrays(Columns, dtype=dtype, shape=shape, 
-			formats = formats, names=names, titles=titles, aligned=aligned,
-			byteorder=byteorder),metadata]
-
+    return [utils.fromarrays(Columns, type = np.ndarray, dtype=dtype, 
+    shape=shape, formats = formats, names=names, titles=titles, 
+    aligned=aligned, byteorder=byteorder), metadata]
 
 def loadSVcols(fname, usecols=None,valuefixer=None, **kwargs):
 
