@@ -1,11 +1,12 @@
 """
-Spreadsheet-style functions for NumPy recarray objects:
+Spreadsheet-style functions for NumPy ndarray with structured dtype or
+recarray objects:
 
 aggregate, aggregate_in, pivot, addrecords, addcols, deletecols, renamecol, 
 replace, colstack, rowstack, nullvalue
 
 Note that these functions are also wrapped as methods of the tabular tabarray 
-object, which is a subclass of the numpy recarray.
+object, which is a subclass of the numpy ndarray.
 
 **See Also:**
 
@@ -27,7 +28,8 @@ from tabular.colors import GrayScale
 
 def aggregate(X, On=None, AggFuncDict=None, AggFunc=None, returnsort=False):
     """
-    Aggregate a recarray on columns for given functions.
+    Aggregate a ndarray with structured dtype (or recarray) on columns for 
+    given functions.
 
     Aggregate a numpy recarray (or tabular tabarray) on a set of specified 
     factors, using specified aggregation functions.
@@ -73,9 +75,9 @@ def aggregate(X, On=None, AggFuncDict=None, AggFunc=None, returnsort=False):
 
     **Parameters**
 
-            **X** :  numpy recarray
+            **X** :  numpy ndarray with structured dtype or recarray
 
-                    The numpy recarray data set to aggregate.
+                    The data set to aggregate.
 
             **On** :  list of  strings, optional
 
@@ -110,7 +112,7 @@ def aggregate(X, On=None, AggFuncDict=None, AggFunc=None, returnsort=False):
 
     **Returns**
 
-            **agg** :  numpy recarray
+            **agg** :  numpy ndarray with structured dtype
 
                     Aggregated data set.
 
@@ -196,15 +198,16 @@ def aggregate(X, On=None, AggFuncDict=None, AggFunc=None, returnsort=False):
     Columns = [ColDict[n] for n in names]
 
     if returnsort:
-        return [np.rec.fromarrays(Columns, names=names),index_array]
+        return [utils.fromarrays(Columns, type=np.ndarray,names=names),index_array]
     else:
-        return np.rec.fromarrays(Columns, names=names)
+        return utils.fromarrays(Columns, type=np.ndarray,names=names)
 
 
 def aggregate_in(Data, On=None, AggFuncDict=None, AggFunc=None, 
                  interspersed=True):
     """
-    Aggregate a recarray and include original data in the result.
+    Aggregate a ndarray with structured dtype or recarray
+    and include original data in the result.
 
     Take aggregate of data set on specified columns, then add the resulting 
     rows back into data set to make a composite object containing both original 
@@ -212,7 +215,7 @@ def aggregate_in(Data, On=None, AggFuncDict=None, AggFunc=None,
 
     First read comments for :func:`tabular.spreadsheet.aggregate`.
 
-    This function returns a numpy recarray, with the number of rows equaling::
+    This function returns a numpy ndarray, with the number of rows equaling::
 
             len(Data) + len(A)
 
@@ -249,9 +252,9 @@ def aggregate_in(Data, On=None, AggFuncDict=None, AggFunc=None,
 
     **Parameters**
 
-            **Data** :  numpy recarray
+            **Data** :  numpy ndarray with structured dtype or recarray
 
-                    The numpy recarray data set to aggregate in.
+                    The data set to aggregate in.
 
             **On** :  list of  strings, optional
 
@@ -288,7 +291,7 @@ def aggregate_in(Data, On=None, AggFuncDict=None, AggFunc=None,
 
     **Returns**
 
-            **agg** :  numpy recarray
+            **agg** :  numpy ndarray with structured dtype
 
                     Composite aggregated data set plus original data set.
 
@@ -318,8 +321,8 @@ def aggregate_in(Data, On=None, AggFuncDict=None, AggFunc=None,
     NewAggregates = aggregate(X, On, AggFuncDict=AggFuncDict, AggFunc=AggFunc)
     on = ','.join(On)
     NewAggregates = addcols(NewAggregates,   
-                            np.rec.fromarrays([[on]*len(NewAggregates)], 
-                                              names=['__aggregates__']))
+                            utils.fromarrays([[on]*len(NewAggregates)], type=np.ndarray,
+                                                                      names=['__aggregates__']))
     AggVars = utils.uniqify(AggVars + On)
     Aggregates = rowstack([OldAggregates,NewAggregates],mode='nulls')
 
@@ -327,7 +330,7 @@ def aggregate_in(Data, On=None, AggFuncDict=None, AggFunc=None,
     U = np.array(utils.uniqify(ANLen)); U.sort()
     [A,B] = fast.equalspairs(ANLen,U)
     Grays = np.array(grayspec(len(U)))
-    AggColor = np.rec.fromarrays([Grays[A]],names = ['__color__'])
+    AggColor = utils.fromarrays([Grays[A]],type=np.ndarray, names = ['__color__'])
 
     Aggregates = addcols(Aggregates,AggColor)
 
@@ -397,7 +400,7 @@ def grayspec(k):
 
 def pivot(X, a, b, Keep=None, NullVals=None, prefix='_'):
     '''
-    Implements pivoting on numpy recarrays.
+    Implements pivoting on numpy ndarrays (with structured dtype) or recarrays.
 
     See http://en.wikipedia.org/wiki/Pivot_table for information about pivot 
     tables.
@@ -425,9 +428,9 @@ def pivot(X, a, b, Keep=None, NullVals=None, prefix='_'):
 
     **Parameters**
 
-            **X** :  numpy recarray
+            **X** :  numpy ndarray with structured dtype or recarray
 
-                    The numpy recarray data set to pivot.
+                    The  data set to pivot.
 
             **a** : string
 
@@ -458,7 +461,7 @@ def pivot(X, a, b, Keep=None, NullVals=None, prefix='_'):
 
     **Returns**
 
-            **ptable** :  numpy recarray
+            **ptable** :  numpy ndarray with structured dtype
 
                     The resulting pivot table.
 
@@ -504,18 +507,18 @@ def pivot(X, a, b, Keep=None, NullVals=None, prefix='_'):
     Db = len(unique_b)
     if len(X) != Da * Db:
         if list(X.dtype.names).index(a) < list(X.dtype.names).index(b):
-            allvalues = np.rec.fromrecords([(x,y) for x in unique_a for y in 
-                                            unique_b], names=[a,b])
+            allvalues = utils.fromrecords([(x,y) for x in unique_a for y in 
+                                            unique_b], type=np.ndarray,names=[a,b])
         else:
-            allvalues = np.rec.fromrecords([(x,y) for x in unique_b for y in 
-                                            unique_a], names=[b,a])
+            allvalues = utils.fromrecords([(x,y) for x in unique_b for y in 
+                                            unique_a], type=np.ndarray,names=[b,a])
         missingvalues = \
             allvalues[np.invert(fast.recarrayisin(allvalues,X[[a,b]]))]
         if NullVals == None:
             NullVals = {}
-        nullvals = np.rec.fromrecords([[NullVals[o] if o in NullVals.keys() 
+        nullvals = utils.fromrecords([[NullVals[o] if o in NullVals.keys() 
                                         else nullvalue(X[o][0]) for o in 
-                                        othernames]], names=othernames)
+                                        othernames]], type=np.ndarray, names=othernames)
         nullarray = nullvals.repeat(len(missingvalues))
         X = rowstack([X, colstack([missingvalues, nullarray])])
 
@@ -541,7 +544,7 @@ def pivot(X, a, b, Keep=None, NullVals=None, prefix='_'):
             NonTrivials.append(c)
             Cols += [Z[:,i] for i in range(Z.shape[1])]
             names += [bn + '_' + c for bn in bnames]
-    D = np.rec.fromarrays(Cols, names=names)
+    D = utils.fromarrays(Cols,type=np.ndarray,names=names)
 
     coloring = {}
     if Keep != None:
@@ -569,7 +572,7 @@ def pivot(X, a, b, Keep=None, NullVals=None, prefix='_'):
 
 def addrecords(X, new):
     """
-    Append one or more records to the end of a numpy recarray.
+    Append one or more records to the end of a numpy recarray or ndarray .
 
     Can take a single record, void or tuple, or a list of records, voids or 
     tuples.
@@ -579,9 +582,9 @@ def addrecords(X, new):
 
     **Parameters**
 
-            **X** :  numpy recarray
+            **X** :  numpy ndarray with structured dtype or recarray
 
-                    The numpy recarray to add records to.
+                    The array to add records to.
 
             **new** :  record, void or tuple, or list of them
 
@@ -589,9 +592,9 @@ def addrecords(X, new):
 
     **Returns**
 
-            **out** :  numpy recarray
+            **out** :  numpy ndarray with structured dtype
 
-                    New numpy recarray made up of `X` plus the new records.
+                    New numpy array made up of `X` plus the new records.
 
     **See also:**  :func:`tabular.spreadsheet.rowstack`
 
@@ -599,12 +602,12 @@ def addrecords(X, new):
     if isinstance(new, np.record) or isinstance(new, np.void) or \
                                                         isinstance(new, tuple):
         new = [new]
-    return np.append(X, np.rec.fromrecords(new, dtype=X.dtype), axis=0)
+    return np.append(X, utils.fromrecords(new, type=np.ndarray,dtype=X.dtype), axis=0)
 
 
 def addcols(X, cols, names=None):
     """
-    Add one or more columns to a numpy recarray.
+    Add one or more columns to a numpy ndarray.
 
     Technical dependency of :func:`tabular.spreadsheet.aggregate_in`.
 
@@ -613,11 +616,11 @@ def addcols(X, cols, names=None):
 
     **Parameters**
 
-            **X** :  numpy recarray
+            **X** :  numpy ndarray with structured dtype or recarray
 
-                    The numpy recarray to add columns to.
+                    The recarray to add columns to.
 
-            **cols** :  numpy recarray, or list of arrays of columns
+            **cols** :  numpy ndarray, or list of arrays of columns
             
                     Column(s) to add.
 
@@ -628,9 +631,9 @@ def addcols(X, cols, names=None):
 
     **Returns**
 
-            **out** :  numpy recarray
+            **out** :  numpy ndarray with structured dtype
 
-                    New numpy recarray made up of `X` plus the new columns.
+                    New numpy array made up of `X` plus the new columns.
 
     **See also:**  :func:`tabular.spreadsheet.colstack`
 
@@ -645,30 +648,31 @@ def addcols(X, cols, names=None):
                    'Trying to add columns of wrong length.'
             assert names != None and len(cols) == len(names), \
                    'Number of columns to add must equal number of new names.'
-            cols = np.rec.fromarrays(cols,names = names)
+            cols = utils.fromarrays(cols,type=np.ndarray,names = names)
         else:
             assert len(cols) == len(X), 'Trying to add column of wrong length.'
-            cols = np.rec.fromarrays([cols], names=names)
+            cols = utils.fromarrays([cols], type=np.ndarray,names=names)
     else:
         assert isinstance(cols, np.ndarray)
         if cols.dtype.names == None:
-            cols = np.rec.fromarrays([cols], names=names)
+            cols = utils.fromarrays([cols],type=np.ndarray, names=names)
 
     Replacements = [a for a in cols.dtype.names if a in X.dtype.names]
     if len(Replacements) > 0:
         print('Replacing columns', 
               [a for a in cols.dtype.names if a in X.dtype.names])
 
-    return np.rec.fromarrays(
+    return utils.fromarrays(
       [X[a] if a not in cols.dtype.names else cols[a] for a in X.dtype.names] + 
       [cols[a] for a in cols.dtype.names if a not in X.dtype.names], 
+      type=np.ndarray,
       names=list(X.dtype.names) + [a for a in cols.dtype.names 
                                    if a not in X.dtype.names])
 
 
 def deletecols(X, cols):
     """
-    Delete columns from a numpy recarray.
+    Delete columns from a numpy ndarry or recarray.
 
     Can take a string giving a column name or comma-separated list of column 
     names, or a list of string column names.
@@ -678,9 +682,9 @@ def deletecols(X, cols):
 
     **Parameters**
 
-            **X** :  numpy recarray
+            **X** :  numpy recarray or ndarray with structured dtype
 
-                    The numpy recarray from which to delete columns.
+                    The numpy array from which to delete columns.
 
             **cols** :  string or list of strings
 
@@ -690,10 +694,10 @@ def deletecols(X, cols):
 
     **Returns**
 
-            **out** :  numpy recarray
+            **out** :  numpy ndarray with structured dtype
 
-                    New numpy recarray given by `X`, excluding the columns
-                    named in `cols`.
+                    New numpy ndarray with structured dtype
+                    given by `X`, excluding the columns named in `cols`.
 
     """
     if isinstance(cols, str):
@@ -707,16 +711,16 @@ def deletecols(X, cols):
 
 def renamecol(X, old, new):
     """
-    Rename column of a numpy recarray in-place.
+    Rename column of a numpy ndarray with structured dtype, in-place.
 
     Implemented by the tabarray method 
     :func:`tabular.tab.tabarray.renamecol`.
 
     **Parameters**
 
-            **X** :  numpy recarray
+            **X** :  numpy ndarray with structured dtype
 
-                    The numpy recarray for which a column is to be renamed.
+                    The numpy array for which a column is to be renamed.
 
             **old** :  string
 
@@ -740,9 +744,9 @@ def replace(X, old, new, strict=True, cols=None, rows=None):
 
     **Parameters**
 
-            **X** :  numpy recarray
+            **X** :  numpy ndarray with structured dtype
 
-                    Numpy recarray for which in-place replacement of `old` with 
+                    Numpy array for which in-place replacement of `old` with 
                     `new` is to be done.
 
             **old** : string
@@ -829,9 +833,9 @@ def replace(X, old, new, strict=True, cols=None, rows=None):
 
 def rowstack(seq, mode='nulls', nullvals=None):
     '''
-    Vertically stack a sequence of numpy record arrays.
+    Vertically stack a sequence of numpy ndarrays with structured dtype
 
-    Analog of numpy.vstack for recarrays.
+    Analog of numpy.vstack
 
     Implemented by the tabarray method
     :func:`tabular.tab.tabarray.rowstack` which uses 
@@ -864,9 +868,9 @@ def rowstack(seq, mode='nulls', nullvals=None):
 
     **Returns**
 
-            **out** :  numpy recarray
+            **out** :  numpy ndarray with structured dtype
 
-                    Result of vertically stacking the recarrays in `seq`.
+                    Result of vertically stacking the arrays in `seq`.
 
     **See also:**  `numpy.vstack 
     <http://docs.scipy.org/doc/numpy/reference/generated/numpy.vstack.html>`_.
@@ -890,23 +894,24 @@ def rowstack(seq, mode='nulls', nullvals=None):
                                        for s in seq if s.dtype.names != None]))
             formats = [max([s.dtype[att] for s in seq if s.dtype.names != None 
                        and att in s.dtype.names]).str for att in names]
-            return np.rec.fromarrays([utils.listunion([s[att].tolist() 
+            return utils.fromarrays([utils.listunion([s[att].tolist() 
                         if (s.dtype.names != None and att in s.dtype.names) 
                         else [nullvals(format)] * len(s) for s in seq]) 
-                        for (att, format) in zip(names, formats)], names=names)
+                        for (att, format) in zip(names, formats)], type=np.ndarray,
+                        names=names)
         elif mode == 'commons':
             names = [x for x in seq[0].dtype.names 
                      if all([x in l.dtype.names for l in seq[1:]])]
             formats = [max([a.dtype[att] for a in seq]).str for att in names]
-            return np.rec.fromrecords(utils.listunion(
-                    [ar.tolist() for ar in seq]), names=names, formats=formats)
+            return utils.fromrecords(utils.listunion(
+                    [ar.tolist() for ar in seq]), type=np.ndarray,names=names, formats=formats)
     else:
         return seq[0]
 
 
 def colstack(seq, mode='abort'):
     """
-    Horizontally stack a sequence of numpy record arrays.
+    Horizontally stack a sequence of numpy ndarrays with structured dtypes
 
     Analog of numpy.hstack for recarrays.
 
@@ -916,7 +921,7 @@ def colstack(seq, mode='abort'):
 
     **Parameters**
 
-            **seq** :  sequence of numpy recarrays
+            **seq** :  sequence of numpy ndarray with structured dtype
 
                     List, tuple, etc. of numpy recarrays to stack vertically.
 
@@ -944,9 +949,9 @@ def colstack(seq, mode='abort'):
 
     **Returns**
 
-            **out** :  numpy recarray
+            **out** :  numpy ndarray with structured dtype
 
-                    Result of horizontally stacking the recarrays in `seq`.
+                    Result of horizontally stacking the arrays in `seq`.
 
     **See also:**  `numpy.hstack 
     <http://docs.scipy.org/doc/numpy/reference/generated/numpy.hstack.html>`_.
@@ -972,17 +977,18 @@ def colstack(seq, mode='abort'):
                         if len(L) > 1 else [(L[0], x)] for (L, x) in NameList])
     else:
         Names = [(L[0], x) for (x, L) in NameList]
-    return np.rec.fromarrays([seq[i][x] for (i, x) in Names], 
-                              names=zip(*Names)[1])
+    return utils.fromarrays([seq[i][x] for (i, x) in Names], 
+                 type= np.ndarray,names=zip(*Names)[1])
 
 
 def join(L, keycols=None, nullvals=None, renamer=None, 
          returnrenaming=False, Names=None):
     """
-    Combine two or more numpy recarrays on common key column(s).
+    Combine two or more numpy ndarray with structured dtype on common key 
+    column(s).
 
-    Merge a list (or dictionary) of numpy recarrays, given by `L`, on key 
-    columns listed in `keycols`.
+    Merge a list (or dictionary) of numpy ndarray with structured dtype, given
+    by `L`, on key columns listed in `keycols`.
 
     This function is actually a wrapper for 
     :func:`tabular.spreadsheet.strictjoin`.
@@ -1084,10 +1090,10 @@ def join(L, keycols=None, nullvals=None, renamer=None,
 
     **Returns**
 
-            **result** :  numpy recarray
+            **result** :  numpy ndarray with structured dtype
 
                     Result of the join, e.g. the result of merging the input
-                    numpy recarrays defined in `L` on the key columns listed in 
+                    numpy arrays defined in `L` on the key columns listed in 
                     `keycols`.
 
             **renaming** :  dictionary of dictionaries, optional
@@ -1169,9 +1175,10 @@ def join(L, keycols=None, nullvals=None, renamer=None,
 
 def strictjoin(L, keycols, nullvals=None, renaming=None, Names=None):
     """
-    Combine two or more numpy recarrays on common key column(s).
+    Combine two or more numpy ndarray with structured dtypes on common key
+    column(s).
 
-    Merge a list (or dictionary) of numpy recarrays, given by `L`, on key 
+    Merge a list (or dictionary) of numpy arrays, given by `L`, on key 
     columns listed in `keycols`.
 
     The ``strictjoin`` assumes the following restrictions:
@@ -1258,10 +1265,10 @@ def strictjoin(L, keycols, nullvals=None, renaming=None, Names=None):
 
     **Returns**
 
-            **result** :  numpy recarray
+            **result** :  numpy ndarray with structured dtype
 
                     Result of the join, e.g. the result of merging the input
-                    numpy recarrays defined in `L` on the key columns listed in 
+                    numpy arrays defined in `L` on the key columns listed in 
                     `keycols`.
 
     **See Also:**
@@ -1356,7 +1363,7 @@ def RenamingIsInCorrectFormat(renaming, L, Names=None):
 
 def DEFAULT_RENAMER(L, Names=None):
     """
-    Renames overlapping column names of numpy recarrays.
+    Renames overlapping column names of numpy ndarrays with structured dtypes
 
     Rename the columns by using a simple convention:
 
