@@ -2,7 +2,7 @@
 Class and functions pertaining to the tabular.tabarray class.
 
 The :class:`tabarray` class is a column-oriented hierarchical data object and 
-subclass of `numpy.recarray <http://docs.scipy.org/doc/numpy/reference/generated/numpy.recarray.html?highlight=recarray#numpy.recarray>`_.
+subclass of `numpy.ndarray <http://docs.scipy.org/doc/numpy/reference/generated/numpy.ndarray.html?highlight=ndarray#numpy.ndarray>`_.
 
 The basic structure of this module is that it contains:
 
@@ -21,7 +21,7 @@ The basic structure of this module is that it contains:
 import os
 import csv
 
-import numpy
+import numpy as np
 
 import tabular.io as io
 import tabular.spreadsheet as spreadsheet
@@ -185,13 +185,13 @@ def tab_join(ToMerge, keycols=None, nullvals=None, renamer=None,
         return Result
 
 
-class tabarray(numpy.core.records.recarray):
+class tabarray(np.ndarray):
     """
-    Subclass of the numpy recarray with extra structure and functionality.
+    Subclass of the numpy ndarray with extra structure and functionality.
 
-    tabarray is a column-oriented data object based on the numpy recarray (a 
-    tabular data object with named columns, each a uniform Python type), with 
-    added functionality and ability to define named groups of columns.
+    tabarray is a column-oriented data object based on the numpy ndarray with
+    structured dtype, with added functionality and ability to define named 
+    groups of columns.
 
     tabarray supports several i/o methods to/from a number of file formats, 
     including (separated variable) text (e.g. ``.txt``, ``.tsv``, ``.csv``), 
@@ -210,9 +210,10 @@ class tabarray(numpy.core.records.recarray):
     def __new__(subtype, array=None, records=None, columns=None, SVfile=None, 
                 binary=None, HSVfile=None, HSVlist=None, shape=None, 
                 dtype=None, formats=None, names=None, titles=None, 
-                aligned=False, byteorder=None, comments=None, delimiter=None, 
+                aligned=False, byteorder=None, buf=None, offset = 0,
+                strides = None, comments=None, delimiter=None, 
                 lineterminator='\n', escapechar=None, quoting=csv.QUOTE_MINIMAL, 
-                quotechar='"', doublequote=True,skipinitialspace=False,
+                quotechar='"', doublequote=True, skipinitialspace=False,
                 skiprows=0, usecols=None, toload=None, metametadata=None, 
                 namesinheader=True, headerlines=None, valuefixer=None, 
                 linefixer=None, coloring=None, wrap=None, 
@@ -224,17 +225,13 @@ class tabarray(numpy.core.records.recarray):
 
                 Data can be passed to the constructor, or loaded from several 
                 different file formats. The constructor first creates a numpy 
-                recarray. If provided, the constructor adds the **coloring** 
-                attribute, which is a dictionary that represents hierarchical 
-                structure on the columns, (e.g. groups of column names).
+                ndarray with structured dtype. If provided, the constructor 
+                adds the **coloring** attribute, which is a dictionary that 
+                represents hierarchical structure on the columns, (e.g. groups
+                of column names).
 
                 **array** :  two-dimensional arrays (:class:`numpy.ndarray`)
 
-                        Load data to a numpy recarray by calling::
-
-                                numpy.rec.fromrecords(array, dtype, shape,  formats, names, titles, aligned, byteorder)
-
-                        unless creating an empty data structure.
 
                         >>> import numpy
                         >>> x = numpy.array([[1, 2], [3, 4]])
@@ -244,38 +241,17 @@ class tabarray(numpy.core.records.recarray):
                         
                         **See also:**  `numpy.rec.fromrecords <http://docs.scipy.org/doc/numpy/reference/generated/numpy.core.records.fromrecords.html#numpy.core.records.fromrecords>`_
 
-                **records** :  list of records
-
-                        Load data to a numpy recarray by calling::
-
-                                numpy.rec.fromrecords(records, dtype, shape,   
-                                    formats, names, titles, aligned, byteorder)
-
-                        Notes from NumPy about numpy.fromrecords:  The data in
-                        the same field can be heterogeneous, they will be
-                        promoted to the highest data type. This method is
-                        intended for creating smaller record arrays. If used to
-                        create large array without formats defined, e.g.::
-
-                                r = fromrecords([(2,3.,'abc')]*100000)
-
-                        it can be slow. If formats is `None`, then this will
-                        auto-detect formats. Use list of tuples rather than 
-                        list of lists for faster processing.
-
+                **records** :  python list of records (tuples or lists)
+ 
                         >>> tabarray(records=[('bork', 1, 3.5), ('stork', 2, -4.0)], names=['x','y','z'])
                         tabarray([('bork', 1, 3.5), ('stork', 2, -4.0)], 
                               dtype=[('x', '|S5'), ('y', '<i4'), ('z', '<f8')])
 
                         **See also:**  `numpy.rec.fromrecords <http://docs.scipy.org/doc/numpy/reference/generated/numpy.core.records.fromrecords.html#numpy.core.records.fromrecords>`_
 
-                **columns** :  list of one-dimensional numpy arrays 
-
-                        Load data to a numpy recarray by calling::
-
-                                numpy.rec.fromarrays(records, dtype, shape, 
-                                formats, names, titles, aligned, byteorder)
-
+                **columns** :  list of python lists or
+                1-D numpy arrays 
+                
                         Fastest when passed a list of numpy arrays, rather than
                         a list of lists.
 
@@ -523,20 +499,20 @@ class tabarray(numpy.core.records.recarray):
 
         if not array is None:
             if len(array) > 0:
-                DataObj = numpy.rec.fromrecords(array, dtype=dtype, 
+                DataObj = utils.fromrecords(array, type=np.ndarray, dtype=dtype, 
                           shape=shape, formats=formats, names=names, 
                           titles=titles, aligned=aligned, byteorder=byteorder)
             else:
-                DataObj = numpy.rec.fromarrays([[]]*len(array.dtype), 
+                DataObj = utils.fromarrays([[]]*len(array.dtype), type=np.ndarray,
                           dtype=dtype, shape=shape, formats=formats, 
                           names=names, titles=titles, aligned=aligned, 
                           byteorder=byteorder)
         elif not records is None:
-            DataObj = numpy.rec.fromrecords(records, dtype=dtype, shape=shape, 
+            DataObj = utils.fromrecords(records, type=np.ndarray, dtype=dtype, shape=shape, 
                       formats=formats, names=names, titles=titles, 
                       aligned=aligned, byteorder=byteorder)
         elif not columns is None:
-            DataObj = numpy.rec.fromarrays(columns, dtype=dtype, shape=shape, 
+            DataObj = utils.fromarrays(columns,type=np.ndarray, dtype=dtype, shape=shape, 
                       formats=formats, names=names, titles=titles, 
                       aligned=aligned, byteorder=byteorder)
         elif not SVfile is None:
@@ -545,7 +521,8 @@ class tabarray(numpy.core.records.recarray):
             [DataObj, metadata] = \
                 io.loadSV(fname=SVfile, names=names, dtype=dtype, shape=shape, 
                 formats=formats, titles=titles, aligned=aligned, 
-                byteorder=byteorder, comments=comments, delimiter=delimiter, 
+                byteorder=byteorder, buf=buf,offset=offset,strides=strides,
+                comments=comments, delimiter=delimiter, 
                 lineterminator=lineterminator, escapechar=escapechar,
                 quoting=quoting,quotechar=quotechar,doublequote=doublequote,
                 skipinitialspace=skipinitialspace, skiprows=skiprows, 
@@ -565,7 +542,7 @@ class tabarray(numpy.core.records.recarray):
                 dtype = givendtype
             if (coloring is None) and (not givencoloring is None):
                 coloring = givencoloring
-            DataObj = numpy.rec.fromrecords(DataObj, dtype=dtype, shape=shape,   
+            DataObj = utils.fromrecords(DataObj, type=np.ndarray, dtype=dtype, shape=shape,   
                       formats=formats, names=names, titles=titles, 
                       aligned=aligned, byteorder=byteorder)
         elif not HSVfile is None:
@@ -577,26 +554,26 @@ class tabarray(numpy.core.records.recarray):
                 names = givennames
             if (coloring is None) and (not givencoloring is None):
                 coloring = givencoloring
-            DataObj = numpy.rec.fromarrays(DataObj, dtype=dtype, shape=shape, 
+            DataObj = utils.fromarrays(DataObj, type=np.ndarray, dtype=dtype, shape=shape, 
                       formats=formats, names=names, titles=titles, 
                       aligned=aligned, byteorder=byteorder)
         elif not HSVlist is None:
             [chkExists(x) for x in HSVlist]
             return io.loadHSVlist(flist=HSVlist)
         else:
-            DataObj = numpy.core.records.recarray.__new__(
-                      numpy.core.records.recarray, shape, dtype=dtype, 
+            DataObj = np.core.records.recarray.__new__(
+                      subtype, shape, dtype=dtype, 
                       formats=formats, names=names, titles=titles, 
-                      aligned=aligned, byteorder=byteorder)
-
-        self = numpy.core.records.recarray.__new__(
-               subtype, DataObj.shape, dtype=DataObj.dtype, names=names)
+                      aligned=aligned, byteorder=byteorder, buf=buf, 
+                      offset=offset, strides=strides)
+                      
+        DataObj = DataObj.view(subtype)              
 
         if not coloring is None:
             coloringsInNames = \
-                 list(set(coloring.keys()).intersection(set(self.dtype.names)))
+                 list(set(coloring.keys()).intersection(set(DataObj.dtype.names)))
             if len(coloringsInNames) == 0:
-                self.coloring = coloring
+                DataObj.coloring = coloring
             else:
                 print ("Warning:  the following coloring keys,", 
                        coloringsInNames, ", are also attribute (column) names " 
@@ -606,19 +583,15 @@ class tabarray(numpy.core.records.recarray):
                        "same names.")
                 for c in coloringsInNames:
                     coloring.pop(c)
-                self.coloring = coloring
+                DataObj.coloring = coloring
         else:
-            self.coloring = {}
+            DataObj.coloring = {}
 
         if not wrap is None:
-            self.coloring[wrap] = self.dtype.names
+            DataObj.coloring[wrap] = DataObj.dtype.names
 
-        if not self.dtype.names is None:
-            for attrib in self.dtype.names:
-                self[attrib] = DataObj[attrib]
-            return self
-        else:
-            return DataObj
+        return DataObj
+        
 
     def __array_finalize__(self, obj):
         """
@@ -638,7 +611,7 @@ class tabarray(numpy.core.records.recarray):
         numerical matrix and do matrix operations.
 
         """
-        return numpy.vstack([self[r] for r in self.dtype.names]).T.squeeze()
+        return np.vstack([self[r] for r in self.dtype.names]).T.squeeze()
 
     def __getitem__(self, ind):
         """
@@ -659,7 +632,7 @@ class tabarray(numpy.core.records.recarray):
 
         *	If you pass a :class:`numpy.ndarray`, you get a tabarray consisting 
         	a subrectangle of the tabarray, as handled by  
-        	:func:`numpy.core.recarray.__getitem__`:
+        	:func:`numpy.ndarray.__getitem__`:
 
                 *	if you pass a 1D NumPy ndarray of booleans of `len(self)`,    
                 	the rectangle contains copies of the rows for which the 
@@ -679,8 +652,8 @@ class tabarray(numpy.core.records.recarray):
                                           else self.coloring[a] for a in ind]))
             return self[ns]
         else:
-            D = numpy.core.records.recarray.__getitem__(self,ind)
-            if isinstance(D,numpy.ndarray) and not D.dtype.names is None:
+            D = np.ndarray.__getitem__(self,ind)
+            if isinstance(D,np.ndarray) and not D.dtype.names is None:
                 D = D.view(tabarray)
                 D.coloring = dict([(k, 
                 list(set(self.coloring[k]).intersection(set(D.dtype.names)))) 
@@ -997,7 +970,7 @@ class tabarray(numpy.core.records.recarray):
         **See also:** :func:`tabular.spreadsheet.join`, :func:`tab_join`
         """
 
-        if isinstance(ToMerge,numpy.ndarray):
+        if isinstance(ToMerge,np.ndarray):
             ToMerge = [ToMerge]
 
         if isinstance(ToMerge,dict):
@@ -1081,9 +1054,9 @@ class tabarray(numpy.core.records.recarray):
                         array([0, 1])
 
         """
-        index_array = numpy.core.fromnumeric._wrapit(self, 'argsort', axis, 
+        index_array = np.core.fromnumeric._wrapit(self, 'argsort', axis, 
                                                      kind, order)
-        index_array = index_array.view(numpy.ndarray)
+        index_array = index_array.view(np.ndarray)
         return index_array
 
 def chkExists( path ):
